"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var fs = _interopRequireWildcard(require("fs"));

var path = _interopRequireWildcard(require("path"));

var _babelPluginMacros = require("babel-plugin-macros");

var _regexgen = _interopRequireDefault(require("regexgen"));

var _package = _interopRequireDefault(require("../package.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

const flatMap = (iteratee, arr) => [].concat(...arr.map(iteratee));

const strToWords = str => str.trim().split(/\s+/);

const pathToWords = path => {
  if (path.isStringLiteral()) {
    return strToWords(path.node.value);
  }

  if (path.isTemplateLiteral()) {
    return strToWords(path.node.quasis[0].value.cooked);
  }

  if (path.isArrayExpression()) {
    return path.get('elements').map(element => {
      if (element.isStringLiteral()) {
        return element.node.value.trim();
      }

      if (element.isTemplateLiteral()) {
        return element.node.quasis[0].value.cooked.trim();
      }

      throw new _babelPluginMacros.MacroError(`${_package.default.name} can't handle ${element.node.type} as array element.`);
    });
  }

  if (path.isObjectExpression()) {
    return path.get('properties').map(property => {
      if (property.node.computed) {
        throw new _babelPluginMacros.MacroError(`${_package.default.name} can't handle computed object properties.`);
      }

      const key = property.get('key');

      if (key.isStringLiteral()) {
        return key.node.value.trim();
      }

      if (key.isIdentifier()) {
        return key.node.name;
      }

      throw new _babelPluginMacros.MacroError(`${_package.default.name} can't handle ${key.node.type} as object property.`);
    });
  }

  if (path.isIdentifier()) {
    const identifierPath = path.scope.getBinding(path.node.name).path;

    if (!identifierPath.isVariableDeclarator()) {
      throw new _babelPluginMacros.MacroError(`${_package.default.name} can only handle declared variables.`);
    }

    return getWords([identifierPath.get('init')]);
  }

  throw new _babelPluginMacros.MacroError(`${_package.default.name} can't handle ${path.node.type}.`);
};

const getWords = paths => flatMap(pathToWords, paths);

const isRegExpConstructor = path => path.isNewExpression() && path.get('callee').node.name === 'RegExp';

const isInRegExpConstructor = path => isRegExpConstructor(path.find(p => isRegExpConstructor(p) || p.isStatement()));

const tryConcatenating = (path, concatenated) => {
  let _path = path,
      key = _path.key;
  path = path.parentPath;

  do {
    if (path.isBinaryExpression()) {
      if (key === 'left') {
        const right = path.get('right');

        if (!right.isStringLiteral()) {
          return null;
        }

        concatenated += right.node.value;
      } else {
        const left = path.get('left');

        if (!left.isStringLiteral()) {
          return null;
        }

        concatenated = left.node.value + concatenated;
      }
    } else if (path.isTemplateLiteral()) {
      if (key !== 0) {
        return null;
      }

      concatenated = path.get('quasis')[0].node.value.cooked + concatenated + path.get('quasis')[1].node.value.cooked;
    } else {
      return null;
    }

    ;
    var _path2 = path;
    key = _path2.key;
    path = path.parentPath;
  } while (!isRegExpConstructor(path));

  return concatenated;
};

const regexgenMacro = ({
  references,
  state: {
    file: {
      opts: {
        filename
      }
    }
  },
  babel: {
    types: t
  }
}) => {
  const usedReferences = Object.keys(references);

  if (usedReferences.length > 1 || usedReferences[0] !== 'default') {
    throw new _babelPluginMacros.MacroError(`${_package.default.name} must be used as default import, instead you have used it as: ${usedReferences.join(', ')}.`);
  }

  references.default.forEach(({
    parentPath: regexgenCall
  }) => {
    if (!regexgenCall.isCallExpression()) {
      throw new _babelPluginMacros.MacroError(`${_package.default.name} should be used as function call, instead you have used it as part of ${regexgenCall.node.type}.`);
    }

    const words = getWords(regexgenCall.get('arguments'));
    const compiledRegExp = (0, _regexgen.default)(words).source;

    if (isInRegExpConstructor(regexgenCall)) {
      const concatenated = tryConcatenating(regexgenCall, compiledRegExp);

      if (concatenated) {
        const regExpConstructor = regexgenCall.find(p => isRegExpConstructor(p));
        const flags = regExpConstructor.get('arguments.1');
        regExpConstructor.replaceWith(t.regExpLiteral(concatenated, flags && flags.node.value));
        return;
      }

      regexgenCall.replaceWith(t.stringLiteral(compiledRegExp));
      return;
    }

    regexgenCall.replaceWith(t.regExpLiteral(compiledRegExp));
  });
};

var _default = (0, _babelPluginMacros.createMacro)(regexgenMacro);

exports.default = _default;